pub const fn null_mut<T: Thin + ?Sized>() -> User<'static, *mut T>Available on crate feature
full only.Expand description
Produces a null User<*mut T>
This will always return an error when writing to
pub const fn null_mut<T: Thin + ?Sized>() -> User<'static, *mut T>full only.Produces a null User<*mut T>
This will always return an error when writing to